//
//  PushReceiver.m
//

#import "PushReceiver.h"

typedef void(^PushReceiverExtLoadHandler)(BOOL success, NSString *richData, NSError *error);

@interface PushReceiver () <PushManagerDelegate>

@end

@implementation PushReceiver

- (id)init {
    self = [super init];
    if (self) {
#ifdef DEBUG
        /* APNS_MODE설정을 위한 값(4.0이상)
         * 여러 위치 사용 가능 설명을 위해 PushReceiver.m 파일에도 샘플 코드가 있음 *
         - APNS_MODE는 서버에서 메시지 발송시 APNS서버를 TEST로 발송할지 REAL로 발송할지의 대한 여부값이 됨
         - RegisterService API호출시 서버에 해당값을 전달
         - PushManager 초기화시 값이 설정되며 default 값은 REAL
         - 개발용 인증서로 빌드시 APNS TOKEN값은 TEST토큰값이므로 DEV설정 필요
         */
        [[PushManager defaultManager].info changeMode:@"DEV"];
#endif
        //NO일 경우엔 PushManager에서 메시지를 가지고 있다가 YES가 되는 시점에 딜리게이트로 전달 처리
        //setEnabled API로 값을 변경처리
        [PushManager defaultManager].enabled = NO;
        //iOS 10 이상 UNUserNotificationCenter Delegate를 PushManager로 설정하는 API
        [[PushManager defaultManager] setUserNotificationDelegate];
    }
    return self;
}

- (void)manager:(PushManager *)manager didLoadPushInfo:(PushManagerInfo *)pushInfo {
    NSLog( @"PushReceiver - manager didLoadPushInfo: %@", pushInfo );
}

- (void)managerDidRegisterForRemoteNotifications:(PushManager *)manager userInfo:(NSDictionary *)userInfo {
    NSLog( @"PushReceiver - managerDidRegisterForRemoteNotifications userInfo: %@", userInfo );
}

- (void)manager:(PushManager *)manager didFailToRegisterForRemoteNotificationsWithError:(NSError *)error {
    NSLog( @"PushReceiver - didFailToRegisterForRemoteNotificationsWithError error: %@", error );
}

- (void)manager:(PushManager *)manager userNotificationCenter:(UNUserNotificationCenter *)center willPresentNotification:(UNNotification *)notification withCompletionHandler:(void (^)(UNNotificationPresentationOptions))completionHandler
{
    completionHandler(UNNotificationPresentationOptionAlert | UNNotificationPresentationOptionBadge | UNNotificationPresentationOptionSound);
}

-(void) didReceiveUserNotification:(NSDictionary *)userInfo status:(NSString *)status messageUID:(NSString *)messageUID {
    NSString *extHTML = [[userInfo objectForKey:@"mps"] objectForKey:@"ext"];
    
    if ( extHTML != nil && [extHTML hasSuffix:@"_ext.html"] ) {
        [self loadExtData:extHTML handler:^(BOOL success, NSString *richData, NSError *error) {
            NSDictionary *notification = [NSMutableDictionary dictionaryWithDictionary:userInfo];
            [[notification objectForKey:@"mps"] setObject:richData forKey:@"ext"];
            
            [self movePushActivityWithNotification:[NSDictionary dictionaryWithDictionary:notification] status:status messageUID:messageUID];
        }];
    }
    else {
        [self movePushActivityWithNotification:userInfo status:status messageUID:messageUID];
    }
}

- (void)manager:(PushManager *)manager didReceiveUserNotification:(NSDictionary *)userInfo status:(NSString *)status messageUID:(NSString *)messageUID {
    
    NSString *extHTML = [[userInfo objectForKey:@"mps"] objectForKey:@"ext"];
    
    if ( extHTML != nil && [extHTML hasSuffix:@"_ext.html"] ) {
        [self loadExtData:extHTML handler:^(BOOL success, NSString *richData, NSError *error) {
            NSDictionary *notification = [NSMutableDictionary dictionaryWithDictionary:userInfo];
            [[notification objectForKey:@"mps"] setObject:richData forKey:@"ext"];
            
            [self movePushActivityWithNotification:[NSDictionary dictionaryWithDictionary:notification] status:status messageUID:messageUID];
        }];
    }
    else {
        [self movePushActivityWithNotification:userInfo status:status messageUID:messageUID];
    }
}

- (void)loadExtData:(NSString *)extHTML handler:(PushReceiverExtLoadHandler)handler {
    
    NSURL *url = [NSURL URLWithString:extHTML];
    
    if (!url) {
        handler(NO, extHTML, nil);
        return;
    }
    
    [NSURLConnection sendAsynchronousRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:extHTML]]
                                       queue:[NSOperationQueue mainQueue]
                           completionHandler:^(NSURLResponse *response, NSData *data, NSError *connectionError) {
                               
                               if ( connectionError != nil ) {
                                   dispatch_async(dispatch_get_main_queue(), ^{
                                       handler(NO, extHTML, connectionError);
                                   });
                                   return;
                               }
                               
                               NSHTTPURLResponse* httpResponse = (NSHTTPURLResponse *)response;
                               
                               if ( httpResponse.statusCode != 200 ) {
                                   dispatch_async(dispatch_get_main_queue(), ^{
                                       handler(NO, extHTML, nil);
                                   });
                                   return;
                               }
                               
                               NSStringEncoding textEncoding = ( [response textEncodingName] ) ? [response textEncodingName].encodingValue : NSUTF8StringEncoding;
                               
                               NSString *result = [[NSString alloc] initWithData:data encoding:textEncoding];
                               NSString *richData = [NSString stringWithString:result];
                               //richData = [richData stringByReplacingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
                               
#if ! __has_feature(objc_arc)
                               [result release];
#endif
                               
                               dispatch_async(dispatch_get_main_queue(), ^{
                                   handler(YES, richData, nil);
                               });
                           }];
}

- (void)movePushActivityWithNotification:(NSDictionary *)notification status:(NSString *)status messageUID:(NSString *)messageUID {
    PPNavigationController *navigationController = [PPNavigationController ppNavigationController];
    PPWebViewController *current = (PPWebViewController *)navigationController.currentViewCtrl;
    NSString *payload = [notification jsonString];
    NSString *pushType = @"APNS";
    [current callCbfunction:@"oniOSReceiveNotification" withObjects:@{@"status":status, @"payload":[payload objectFromJsonString], @"type":pushType, @"messageUID": messageUID}, nil];
}


@end
